
%% segment the palm
%
%% input
%I - input depth image (in mm expeted),
%invalid\too far\too near pixels are supposed to be set to 0
%options - (if no options specified, the default values are used)

%% options (*==optional):
%
%*calibparam - camera calibration parameters
%*maxSize - maximal hand size (default 180)
%*imgBorder - image border (number of pixels from image border, that should
%not be taken into account (default - no border) 
%*thrSpace - region growing threashold parameter (default 10)
%*verbose - should  information be shown (default false)

%% output:
%segnew - final segmentation
%seg - initial segmentation
%p - the nearest point

function [segnew,seg,p] = segmentHand(I,options)

    if (~exist('options','var'))
        options = struct();
    end

    if isfield(options,'calibparam')
        calibparam = options.calibparam;
    else
        calibparam = [];
    end
    
    if ~isfield(options,'maxSize')
        options.maxSize = 180; %for RG 
    end

    if isfield(options,'imgBorder')
        imgBorder = options.imgBorder; 
    else
        imgBorder = [];
    end
    
    if ~isfield(options,'thrSpace')
        options.thrSpace = 10;
    end
    
    if ~isfield(options,'verbose')
        options.verbose = 0;
    end
    
    %cut off object border pixels - they are noisy
    mask = (I == 0);
    se = strel('disk', 5);
    mask = imdilate(mask,se);
   
    %add borders 
    if (~isempty(imgBorder))
        mask(1:imgBorder,:) = 1;
        mask(end-imgBorder+1:end,:)=1;
        mask(:,1:imgBorder) =1;
        mask(:,end-imgBorder+1:end) = 1;
    end
    
    If = medfilt2(I,[8,8]).*(I>0);
    Ifc = If.*(~mask); %If, possibly cropped and without borders
    
    minval = min(Ifc(Ifc>0));
    
    [x,y]=find(Ifc == minval);
    p = [x y];
    
    options.maxSize = 0.8*options.maxSize; 
    seg = regionGrowing(If,ones(size(I)),p,options);
    %seg = (region==1) & (If - If(x(1),y(1)) < 0.8*maxHandSize); %needed?
    options.maxSize = options.maxSize/0.8;
    
    %postprocess
    se = strel('disk', 3);
    seg = imopen(seg,se);
    
    %there was something segmented - cut the parts that are not needed
    if (sum(sum(seg))>1 && ~isempty(calibparam))
    
        segnew = handSizeCheck(If,seg,options);    
        segnew = segnew.*(I>0);
       
    else
        
        segnew = seg;
    end

end